@@ -1,8 +1,14 @@
+{-# LANGUAGE CPP #-}
+
 -- | Word8 library to be used with Data.ByteString.
 -- All function assumes that 'Word8' is encoded in Latin-1 (ISO-8859-1).
 -- All utility functions are supposed to work as if
 -- those of 'Data.Char'. Exceptions are described in
 -- the function documentations.
+--
+-- Base library 4.7 (GHC 7.8) or earlier is based on Unicode 6.
+-- Base library 4.8 (GHC 7.10) or later is based on Unicode 7.
+-- 'isLower', 'isSymbol' and 'isPunctuation' behave differently.
 
 module Data.Word8 (
   -- * Re-exporting
@@ -34,6 +40,10 @@ module Data.Word8 (
 
 import Data.Word (Word8)
 
+#ifndef MIN_VERSION_base
+#define MIN_VERSION_base(x,y,z) 1
+#endif
+
 ----------------------------------------------------------------
 
 isControl :: Word8 -> Bool
@@ -49,10 +59,20 @@ isSpace w = w == _space
          || w == _vt
          || w == _nbsp
 
+-- | This function returns 'True' for 170 and 186 in Unicode 6.
+--   But it returns 'False' in Unicode 7.
 isLower :: Word8 -> Bool
 isLower w = isLower' w
-         || w == _ordfeminine
          || w == _mu
+#if !MIN_VERSION_base(4,8,0)
+         || w == _ordfeminine
+         || w == _ordmasculine
+#endif
+
+isLowerCommon :: Word8 -> Bool
+isLowerCommon w = isLower' w
+         || w == _mu
+         || w == _ordfeminine
          || w == _ordmasculine
 
 isLower' :: Word8 -> Bool
@@ -66,7 +86,7 @@ isUpper w = isAsciiUpper w
          || _Oslash <= w && w <= _Thorn
 
 isAlpha :: Word8 -> Bool
-isAlpha w = isLower w || isUpper w
+isAlpha w = isLowerCommon w || isUpper w
 
 isAlphaNum :: Word8 -> Bool
 isAlphaNum w = isAlpha w || isNumber w
@@ -89,7 +109,7 @@ isHexDigit w = isDigit w
             || _a <= w && w <= _f
 
 isLetter :: Word8 -> Bool
-isLetter w = isLower w || isUpper w
+isLetter w = isLowerCommon w || isUpper w
 
 isMark :: Word8 -> Bool
 isMark _ = False
@@ -103,11 +123,23 @@ isNumber w = isDigit w
           || w == _1'2
           || w == _3'4
 
+-- | This function returns 'False' for 167 and 182 in Unicode 6.
+--   But it returns 'True' in Unicode 7.
 isPunctuation :: Word8 -> Bool
+#if MIN_VERSION_base(4,8,0)
+isPunctuation w = w `elem` [0x21,0x22,0x23,0x25,0x26,0x27,0x28,0x29,0x2a,0x2c,0x2d,0x2e,0x2f,0x3a,0x3b,0x3f,0x40,0x5b,0x5c,0x5d,0x5f,0x7b,0x7d,0xa1,0xa7,0xab,0xb6,0xb7,0xbb,0xbf]
+#else
 isPunctuation w = w `elem` [0x21,0x22,0x23,0x25,0x26,0x27,0x28,0x29,0x2a,0x2c,0x2d,0x2e,0x2f,0x3a,0x3b,0x3f,0x40,0x5b,0x5c,0x5d,0x5f,0x7b,0x7d,0xa1,0xab,0xb7,0xbb,0xbf]
+#endif
 
+-- | This function returns 'True' for 167 and 182 in Unicode 6.
+--   But it returns 'False' in Unicode 7.
 isSymbol :: Word8 -> Bool
+#if MIN_VERSION_base(4,8,0)
+isSymbol w = w `elem` [0x24,0x2b,0x3c,0x3d,0x3e,0x5e,0x60,0x7c,0x7e,0xa2,0xa3,0xa4,0xa5,0xa6,0xa8,0xa9,0xac,0xae,0xaf,0xb0,0xb1,0xb4,0xb8,0xd7,0xf7]
+#else
 isSymbol w = w `elem` [0x24,0x2b,0x3c,0x3d,0x3e,0x5e,0x60,0x7c,0x7e,0xa2,0xa3,0xa4,0xa5,0xa6,0xa7,0xa8,0xa9,0xac,0xae,0xaf,0xb0,0xb1,0xb4,0xb6,0xb8,0xd7,0xf7]
+#endif
 
 isSeparator :: Word8 -> Bool
 isSeparator w = w == _space
